/* e_elem2.cc
 *$Header: /al/acs/src/RCS/e_elem2.cc,v 7.98 95/04/19 01:05:47 al Exp $
 * Base class for elements of a circuit
 */
#include "ecah.h"
#include "ap.h"
#include "error.h"
#include "e_elemnt.h"
#include "u_option.h"
#include "u_status.h"
#include "io.h"
#include "s__.h"
#include "declare.h"
/*--------------------------------------------------------------------------*/
//	void	ELEMENT::parse(CS&);
//	void	ELEMENT::print(int,int)const;
//	void	ELEMENT::expand_pre();
//	void	ELEMENT::trsetup();
//	int	ELEMENT::conv_check()const;
/*--------------------------------------------------------------------------*/
void ELEMENT::parse(CS& cmd)
{
  parselabel(cmd);
  parsenodes(cmd,2);
  parseexpr(cmd);
}
/*--------------------------------------------------------------------------*/
void ELEMENT::print(int where, int detail)const
{
  printlabel(where);
  printnodes(where);
  printexpr(where);
  mputc('\n', where);
}
/*--------------------------------------------------------------------------*/
void ELEMENT::expand_pre()
{
  switch (method_u){
  case mGEAR:
  case mTRAPEZOID:
    method_a = method_u;
    break;
  case mSTIFF:
    method_a = mGEAR;
    break;
  default:
    method_a = OPT::method;
    break;
  }
}
/*--------------------------------------------------------------------------*/
/* trsetup: set up guesses for model evaluation
 * saves old model inputs, and computes new inputs (volts and current)
 * also saves old matrix parameters, for convergence checking
 */
void ELEMENT::trsetup()
{
  y2 = y1;
  y1 = y0;
  if (SIM::mode == sDC  ||  SIM::phase == pINIT_DC){	// initial dc
    mt1 = m0;
    time0 = SIM::time0;
    time1 = time2 = time3 = 0.;
  }else if (time0 != SIM::time0){	// new time step
    if (time0 < SIM::time0){		// forward
      mt1 = m0;
      yt3 = yt2;
      yt2 = yt1;
      yt1 = y0;
      time3 = time2;
      time2 = time1;
      time1 = time0;
    }					// else backward, don't save
    time0 = SIM::time0;
  }
}
/*--------------------------------------------------------------------------*/
/* conv_check: check branch for convergence
 * should be a macro for speed
 */
int ELEMENT::conv_check()const
{
  int isconverged = conchk(y1.f0,y0.f0,OPT::abstol);
  if ((STATUS::iter[iSTEP] >= OPT::itl[8]) && !isconverged){
    error(bPICKY,"%s: non-convergence: (f0) (%s,%s,%s)\n",
          printlabel(),
          ftos(y0.f0, "", 7, 0),
          ftos(y1.f0, "", 7, 0),
          ftos(y2.f0, "", 7, 0));
    if (!conchk(y1.f1,y0.f1,OPT::abstol)){
      error(bPICKY,"%s: non-convergence: (f1) (%s,%s,%s)\n",
	    printlabel(),
	    ftos(y0.f1, "", 7, 0),
	    ftos(y1.f1, "", 7, 0),
	    ftos(y2.f1, "", 7, 0));
    }else{
      error(bPICKY,"%s: f1 is ok\n", printlabel());
    }
  }

  if (isconverged){
    isconverged = conchk(y1.f1,y0.f1,OPT::abstol);
    if ((STATUS::iter[iSTEP] >= OPT::itl[8]) && !isconverged){
      error(bPICKY,"%s: non-convergence: (f1 only) (%s,%s,%s)\n",
	    printlabel(),
	    ftos(y0.f1, "", 7, 0),
	    ftos(y1.f1, "", 7, 0),
	    ftos(y2.f1, "", 7, 0));
    }
  }
  return isconverged;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
