/* s_fo_out.cc
 *$Header: /al/acs/src/RCS/s_fo_out.cc,v 7.99 95/04/22 00:25:17 al Exp $
 * functions needed in addition to the transient analysis
 * to perform the fourier command.
 */
#include "ecah.h"
#include "error.h"
#include "io.h"
#include "mode.h"
#include "u_probe.h"
#include "s_fo.h"
#include "declare.h"
/*--------------------------------------------------------------------------*/
//	void	FOURIER::store(void);
//	void	FOURIER::foout(void);
//	void	FOURIER::fohead(int);
//	void	FOURIER::foprint(complex*);
static  int	stepnum(double,double,double);
static	complex	find_max(complex*,int,int);
static	double  phaze(complex);
static	double  db(complex);
/*--------------------------------------------------------------------------*/
extern char e_int[];
/*--------------------------------------------------------------------------*/
/* store: stash time domain data in preparation for Fourier Transform
 */
void FOURIER::store(void)
{
  for (int ii = 0;  ii < printlist.count();  ii++){
    fdata[ii][stepno] = printlist[ii].value();
  }
}
/*--------------------------------------------------------------------------*/
/* foout:  print out the results of the transform
 */
void FOURIER::foout(void)
{
  plclose();
  plclear();
  for (int ii = 0;  ii < printlist.count();  ii++){
    fohead(ii);
    fft(fdata[ii], timesteps-1,  0);
    foprint(fdata[ii]);
  }
}
/*--------------------------------------------------------------------------*/
/* fo_head: print output header
 * arg is index into probe array, to select probe name
 */
void FOURIER::fohead(int prob)
{
 mprintf(IO::where,"# %-10s", printlist[prob].label());
 mprintf(IO::where,"--------- actual ---------  -------- relative --------\n");
 mprintf(IO::where,"#freq       ");
 mprintf(IO::where,"value        dB      phase  value        dB      phase\n");
}
/*--------------------------------------------------------------------------*/
/* fo_print: print results of fourier analysis
 * for all points at single probe
 */
void FOURIER::foprint(complex *data)
{
  int ii;
  int startstep = stepnum(0., fstep, fstart);
  int stopstep  = stepnum(0., fstep, fstop );
  complex maxvalue = find_max(data,MAX(1,startstep),stopstep);
  if (maxvalue == 0.)
    maxvalue = 1.;
  data[0] /= 2;
  for (ii = startstep;  ii <= stopstep;  ++ii){
    double frequency = fstep * ii;
    complex unscaled = data[ii];
    complex scaled = unscaled / maxvalue;
    unscaled *= 2;
    mprintf(IO::where, "%s%s%7.2f %8.3f %s%7.2f %8.3f\n",
	    ftos(frequency,    "           ",5,IO::formaat),
	    ftos(abs(unscaled),"           ",5,IO::formaat),
	    db(unscaled),
	    phaze(unscaled*complex(0.,1)),
	    ftos(abs(scaled),  "           ",5,IO::formaat),
	    db(scaled),
	    phaze(scaled) ) ;
  }
}
/*--------------------------------------------------------------------------*/
/* stepnum: return step number given its frequency or time
 */
static int stepnum(double start, double step, double here)
{
  return int((here-start)/step + .5);
}
/*--------------------------------------------------------------------------*/
/* find_max: find the max magnitude in a complex array
 */
static complex find_max(complex *data, int start, int stop)
{
  complex maxvalue = 0.;
  int ii;
  for (ii = start;  ii <= stop;  ii++){
    if (abs(data[ii]) > abs(maxvalue)){
      maxvalue = data[ii];
    }
  }
  return maxvalue;
}
/*--------------------------------------------------------------------------*/
/* phaze: extract phase (degrees) from complex number
 * rotates 90 degrees!  (ref to sine instead of cosine)
 */
static double phaze(complex x)
{
  return arg(x)* RTOD;
}
/*--------------------------------------------------------------------------*/
static double db(complex value)
{
  return  20. * log10(MAX(abs(value),VOLTMIN));
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
